package cz.upol.foolang.ast;

import java.util.List;
import java.util.Set;

import cz.upol.foolang.types.GeneralType;
import cz.upol.foolang.types.Type;


/**
 * Trida slouzici k vytvareni jednotlivych objektu AST.
 * 
 * Aktualni implementace pouze vola konstruktory jednotlivych uzlu,
 * ale v ramci optimalizaci, lze implementovat i slozitejsi logiku,
 * napr. sdileni objektu. 
 */
public class ASTBuilder {
	
	public Constant createConstant(int value) {
		return new Constant(int.class, value);
	}
	
	public Constant createConstant(double value) {
		return new Constant(double.class, value);
	}
	
	public Constant createConstant(boolean value) {
		return new Constant(boolean.class, value);
	}
	
	public Constant createConstant(Object value) {
		return new Constant(value.getClass(), value);
	}

	public Node createBinaryOperator(Operator operator, Node node1, Node node2) {
		return new BinaryOperator(operator, node1, node2);
	}
	
	public Node createVariable(String name) {
		return new Variable(name);
	}
	
	public Node createReturn(Node node) {
		return new Return(node);
	}

	public Argument createArgument(Type type, String name) {
		return new Argument(type, name);
	}

	public Node createBlock(List<Node> ll) {
		return new Block(ll.toArray(new Node[ll.size()]));
	}
	
	public Node createMethod(String name, Set<Modifier> modifiers, Type returnType, List<Argument> arguments, Node code) {
		return new MethodDeclaration(name, modifiers, returnType, arguments, code);
	}

	public Node createClass(String name, List<Node> declarations) {
		return new ClassDeclaration(name, declarations.toArray(new Node[declarations.size()]));
	}

	public Type createType(List<String> packageHierarchy, String typeName) {
		return new GeneralType(packageHierarchy, typeName);
	}

	public Node createField(String name, Set<Modifier> modifiers, Type type) {
		return new FieldDeclaration(name, modifiers, type);
	}

	public Node createVarDecl(String name, Type type) {
		return new VariableDeclaration(name, type);
	}
}
