package cz.upol.foolang.ast;

import java.util.ArrayList;
import java.util.List;

/**
 * Abstraktni trida predstavujici uzel AST
 * 
 * Kazdy uzel ma sve unikatni id a pripadne potomky.
 * Potomci teto tridy by meli byt immutable.
 */
public abstract class Node {
	private static int lastId = 0;
	
	private final int id;
	private final List<Node> subNodes;
		
	public Node(Node... subNodes) {
		this.id = (++lastId);
		this.subNodes = new ArrayList<Node>(subNodes.length);
		
		for (Node n: subNodes)
			this.subNodes.add(n);
	}
	
	public Iterable<Node> getChildren() {
		return subNodes;
	}
	
	public int getId() {
		return id;
	}
}
