package cz.upol.foolang.types;

import java.util.List;
import java.util.stream.Collectors;

/**
 * Obecny typ, odpovidajici jedne konkretni tride.
 * Predpokladame, ze kazda trida ma sve jmeno a je umistena
 * v nejake balicku.
 */
public class GeneralType implements Type {
	
	private final List<String> typePackage;
	private final String typeName;

	public GeneralType(List<String> typePackage, String typeName) {
		this.typePackage = typePackage;
		this.typeName = typeName;
	}

	public List<String> getTypePackage() {
		return typePackage;
	}

	public String getTypeName() {
		return typeName;
	}

	@Override
	public String toString() {
		return typePackage.stream().collect(Collectors.joining(".")) + "." + getTypeName();
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((typeName == null) ? 0 : typeName.hashCode());
		result = prime * result + ((typePackage == null) ? 0 : typePackage.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		GeneralType other = (GeneralType) obj;
		if (typeName == null) {
			if (other.typeName != null)
				return false;
		} else if (!typeName.equals(other.typeName))
			return false;
		if (typePackage == null) {
			if (other.typePackage != null)
				return false;
		} else if (!typePackage.equals(other.typePackage))
			return false;
		return true;
	}
}
